import pygame, random
from pygame.locals import *

from enemies.enemy import *

armoffsets=[[(1,1),(1,2),(1,3),(1,4)],#north, not being done now
[(0,1),(0,2),(1,4),(1,5)],#east also not being used
[(-3,-4),(-2,-5),(-10,-20),(-3,-4)],#south, also not in use
[(1,-1),(2,0),(2,1),(1,2)],#west, the real deal
]


class DeadBoss(Enemy):
	HeadAnimations=[ [image('kraken/head%s%i.png'%(dir, frame+1)) for frame in range(4)] for dir in dirs]
	booms=[image('bigexplode%i.png'%(i+1)) for i in range(8)]
	
	def __init__(self, game,x,y,dir):
		self.boomsound=pygame.mixer.Sound('../sound/bossdead.ogg')
		Enemy.__init__(self,game,x,y)
		self.dir=dir
		self.shootable=False
		self.explosionframecount=0
	def nextsquare(self):
		return None
	def offset(self):
		return -32,0
	def update(self):
		Enemy.update(self)
		if len(self.game.enemies)==1:
			if self.explosionframecount==0:
				self.boomsound.play()
			self.explosionframecount+=1
			if self.explosionframecount>8*8:
				self.game.boss_victory()
	def image(self):
		if len(self.game.enemies)>1:
			return self.HeadAnimations[self.dir][0]
		else:
			if self.explosionframecount//8>=8:
				return None
			return self.booms[self.explosionframecount//8]
	def offset(self):
		return -32,0
		

class Boss(Enemy):
	HeadAnimations=[ [image('kraken/head%s%i.png'%(dir, frame+1)) for frame in range(4)] for dir in dirs]
	FullAnimations=[ [image('kraken/enter%s%i.png'%(dir, frame+1)) for frame in range(4)] for dir in dirs]
	def __init__(self, game, xstart,ystart,x,y):
		Enemy.__init__(self,game,xstart,ystart)
		self.progressrate=0.5
		self.hp=42
		self.xwait=x
		self.ywait=y
		self.donemoving=False
		self.lastdirection=self.directionto(x,y)
	def nextsquare(self):
		if self.fromsquare.x==self.xwait and self.fromsquare.y==self.ywait:
			if not self.donemoving:
				self.donemoving=True
				self.lastdirection=(self.lastdirection+2)%4
			self.spawnarms()
			return None
		else:
			oldx,oldy=self.fromsquare.x,self.fromsquare.y
			for yadd in range(2):
				for xadd in range(3):
					x,y=oldx+xadd,oldy+yadd
					if self.game.inbounds(x,y):
						self.game.grid[y][x].enemy=self
						if self.game.grid[y][x].haswall:
							self.game.smash(x,y)
				if self.game.inbounds(oldx-1,oldy+yadd):
					self.game.grid[oldy+yadd][oldx-1].enemy=None
			direction=self.directionto(self.xwait,self.ywait)
			result= self.squarefromdirection(direction)
			return result
	def spawnarms(self):
		rotation=0
		for spawnx,spawny in armoffsets[self.direction()]:
			x,y=spawnx+self.fromsquare.x,spawny+self.fromsquare.y
			if self.game.inbounds(x,y) and not self.game.grid[y][x].enemy:
				newdir=self.direction()
				if rotation==0:
					newdir=(newdir-1)%4
				if rotation==3:
					newdir=(newdir+1)%4
				self.game.enemies.append(TentaclePiece(self.game,self,None,x,y,newdir))
				if self.game.grid[y][x].haswall:
					self.game.smash(x,y)
			rotation+=1
	def offset(self):
		if self.donemoving:
			return -32,0
		else:
			return -64,-32
	def death(self):
		for spawnx,spawny in armoffsets[self.direction()]:
			x,y=spawnx+self.fromsquare.x,spawny+self.fromsquare.y
			if self.game.inbounds(x,y) and self.game.grid[y][x].enemy:
				self.game.grid[y][x].enemy.spreaddying()
		self.game.enemies.append(DeadBoss(self.game,self.fromsquare.x,self.fromsquare.y,self.direction()))
	def image(self):
		if self.donemoving:
			return self.grabframe(self.HeadAnimations[self.direction()])
		else:
			return self.grabframe(self.FullAnimations[self.direction()])

def loadconnectors(green=False):
	result=[]
	greentext=''
	if green:
		greentext='g'
	for dir1 in dirs:
		row=[]
		for dir2 in dirs:
			if dir1==dir2:
				row.append(None)
			else:
				val=None
				try:
					val=[image('kraken/tentacle%s%s%s%i.png'%(greentext,dir1,dir2, frame+1)) for frame in range(4)]
				except:
					val=[image('kraken/tentacle%s%s%s%i.png'%(greentext,dir2,dir1, frame+1)) for frame in range(4)]
				row.append(val)
		result.append(row)
	return result

deadtentaclesound=None
def boomsound():
	global deadtentaclesound
	if not deadtentaclesound:
		deadtentaclesound=pygame.mixer.Sound('../sound/armdead.ogg')
	return deadtentaclesound

class TentaclePiece(Enemy):
	endbits= [[image('kraken/tentacle%s%i.png'%(dir, frame+1)) for frame in range(4)] for dir in dirs]
	greenendbits= [[image('kraken/tentacleg%s%i.png'%(dir, frame+1)) for frame in range(4)] for dir in dirs]
	
	connectors=loadconnectors()
	greenconnectors=loadconnectors(True)
	def __init__(self,game,boss, previous,x,y,direction=None):
		Enemy.__init__(self,game,x,y)
		self.boomsound=boomsound()
		self.previous=previous
		self.next=None
		self.boss=boss
		self.charge=0
		self.growth=0
		self.dying=False
		self.deathcount=0
		self.waitingoncharge=False
		self.dir=direction
	def update(self):
		if self.dying:
			if not self.next:
				self.deathcount+=1
				if self.deathcount>5:
					self.game.remove_enemy(self)
					self.fromsquare.enemy=None
					self.boomsound.play()
					self.game.enemies.append(Explosion(self.game,self.fromsquare.x,self.fromsquare.y))
					if self.previous:
						self.previous.next=None
			#hold still while dying
			return
		Enemy.update(self)
		
		if not self.next and not self.waitingoncharge:
			for neighbor in self.game.neighborsquares(self.fromsquare):
				if neighbor.haswall:
					self.waitingoncharge=True
					self.alertcharge()
		
		if self.waitingoncharge and self.charge==0:
			pass
		elif self.charge>0:
			self.charge+=1
			if self.charge>=6:
				if self.next:
					if self.next.charge==0:
						self.next.charge=1
				else:
					x,y=self.xy()
					self.game.boom(x,y)
					self.clearcharge()
		elif not self.next:
			self.growth+=1
			if self.growth>=20:
				#go to pearls
				if random.randint(1,3)==1:
					tx,ty=random.choice(self.game.pearllocations)
					square=self.squarefromdirection(self.directionto(tx,ty))
				else: 
					x,y=self.xy()
					tx,ty=random.choice(self.game.neighbors(x,y))
					if self.game.inbounds(tx,ty):
						square=self.game.grid[ty][tx]
				if square and square.enemy==None:
					self.growth=0
					self.next=TentaclePiece(self.game,self.boss, self, square.x,square.y)
					self.game.enemies.append(self.next)
					if square.haspearl:
						self.game.take_pearl(square)
	
	def clearcharge(self):
		self.charge=0
		self.waitingoncharge=False
		if self.previous:
			self.previous.clearcharge()
	def alertcharge(self):
		if not self.previous:
			self.charge=1
		else:
			self.previous.alertcharge()
	def death(self):
		
		if self.next:
			self.next.spreaddying()
		if self.previous:
			self.previous.next=None
		
	def spreaddying(self):
		self.dying=True
		if self.next:
			self.next.spreaddying()
	
	def nextsquare(self):
		return None
	def image(self):
		ends=self.endbits
		conns=self.connectors
		if self.charge>5:
			ends=self.greenendbits
			conns=self.greenconnectors
		if self.dir is not None:
			if self.next:
				x,y=self.next.xy()
				otherdir=self.directionto(x,y)
				return self.grabframe(conns[self.dir][otherdir])
			else:
				return self.grabframe(ends[self.dir])
		else:
			x,y=self.previous.xy()
			dir=self.directionto(x,y)
			if self.next:
				x,y=self.next.xy()
				otherdir=self.directionto(x,y)
				return self.grabframe(conns[dir][otherdir])
			else:
				return self.grabframe(ends[dir])