import pygame
from pygame.locals import *
from screens.text import TextOverlay
from screens.game import GameScreen, GridSquare, Gun
from enemies.enemy import JellyFish

class Tutorial:	
	def __init__(self, game):
		self.game=game
		self.gamescreen=GameScreen(game,0)
		self.donewelcometext=False
		self.progress=0
		self.input_blocked=False
		self.welcomeMessages=["Welcome to the tutorial!\n(Click to continue.)",
			"In this game you protect your 3 rare pearls. They are\nlocated in the center of the screen.",
			"The evil ocean master is sending his minions to destroy\nyour walls and take your pearls!",
			"You have protected your pearls with a wall. It is not enough.\nBuild another section of wall to continue."]
			
		self.gameplay_frozen=False
		self.donefirstpiecetext=False
		self.test_piece=None
	def update(self):
		if self.progress==0:
			self.gamescreen.nextpiece=[[GridSquare(0,0),GridSquare(0,1),GridSquare(0,2)]]
			self.gamescreen.nextpiece[0][0].haswall=True
			self.gamescreen.nextpiece[0][1].haswall=True
			self.gamescreen.nextpiece[0][2].haswall=True
			self.test_piece=self.gamescreen.nextpiece
			self.game.active_screen=TextOverlay(self.game, self, self.welcomeMessages)
			self.progress=1
		elif self.progress==1 and self.test_piece!=self.gamescreen.nextpiece:
			self.game.active_screen=TextOverlay(self.game,self, ["Good job. Now place the power crystal to create a source\nof energy."])
			self.progress=2
			self.gamescreen.nextpiece=[[GridSquare(0,0),GridSquare(0,1),GridSquare(0,2),GridSquare(1,1),GridSquare(-1,1)]]
			self.gamescreen.nextpiece[0][0].haswall=True
			self.gamescreen.nextpiece[0][1].haswall=True
			self.gamescreen.nextpiece[0][0].ischargable=True
			self.gamescreen.nextpiece[0][1].ischargable=True
			self.gamescreen.nextpiece[0][2].ischargable=True
			self.gamescreen.nextpiece[0][3].ischargable=True
			self.gamescreen.nextpiece[0][4].ischargable=True
			self.gamescreen.nextpiece[0][1].ispowersource=True
			self.gamescreen.nextpiece[0][2].haswall=True
			self.gamescreen.nextpiece[0][3].haswall=True
			self.gamescreen.nextpiece[0][4].haswall=True
			self.crystal_piece=self.gamescreen.nextpiece
		elif self.progress==2 and self.crystal_piece!=self.gamescreen.nextpiece:
			self.game.active_screen=TextOverlay(self.game, self, ["The power crystal provides power to the neighboring\nconnectors.",
				"Now connect the gun piece to the power source to activate it."])
			self.gamescreen.nextpiece=[[GridSquare(0,0),GridSquare(0,1),GridSquare(0,2),GridSquare(1,1),GridSquare(-1,1)]]
			self.gamescreen.nextpiece[0][0].haswall=True
			self.gamescreen.nextpiece[0][1].haswall=True
			self.gamescreen.nextpiece[0][0].ischargable=True
			self.gamescreen.nextpiece[0][1].ischargable=True
			self.gamescreen.nextpiece[0][2].ischargable=True
			self.gamescreen.nextpiece[0][3].ischargable=True
			self.gamescreen.nextpiece[0][4].ischargable=True
			self.gamescreen.nextpiece[0][1].gun=Gun(self.gamescreen)
			self.gamescreen.nextpiece[0][2].haswall=True
			self.gamescreen.nextpiece[0][3].haswall=True
			self.gamescreen.nextpiece[0][4].haswall=True
			self.gunpiece=self.gamescreen.nextpiece
			self.progress=3
		elif self.progress==3 and self.gunpiece!=self.gamescreen.nextpiece and not self.has_active_gun():
			self.game.active_screen=TextOverlay(self.game, self, ["The gun isn't connected to power.",
				"Try again, making sure that the connectors link the gun to a\ncrystal."])
			self.gamescreen.nextpiece=self.gunpiece
			self.gamescreen.nextpiece[0][1].gun=Gun(self.gamescreen)
		elif self.progress==3 and self.gunpiece!=self.gamescreen.nextpiece and self.has_active_gun():
			self.game.active_screen=TextOverlay(self.game, self, ["Good job.  Guns require power to work.",
				"Continue to build and experiment for a little bit."])
			self.progress=4
			self.buildframecounter=0
		elif self.progress==4 and self.buildframecounter<30*15:
			self.buildframecounter+=1
		elif self.progress==4 and self.buildframecounter>=30*15:
			self.game.active_screen=TextOverlay(self.game, self, ["Here come the ocean master's minions.",
				"Survive the assault to complete the tutorial. Don't worry,\nthey will explode against your walls."])
			self.progress=5
			for pos in ( (0,0),(30,0),(0,21),(30,21),(15,0),(15,21) ):
				self.gamescreen.enemies.append(JellyFish(self.gamescreen, pos[0],pos[1],'blue'))
		elif self.progress==5 and len(self.gamescreen.enemies)==0:
			self.game.active_screen=TextOverlay(self.game, GameScreen(self.game,1),
				["Congratulations, you have completed the tutorial!\nTo advance: build turrets and kill enemies."])
		if self.gameplay_frozen:
			return
		else:
			self.gamescreen.update()
	
	def has_active_gun(self):
		for gun in self.gamescreen.guns:
			if gun.square.ischarged:
				return True
		return False
	
	def handle_event(self, event):
		if self.input_blocked:
			return
		else:
			self.gamescreen.handle_event(event)
	def draw(self, window):
		self.gamescreen.draw(window)
	def get_music(self):
		return self.gamescreen.get_music()
		